/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace Kabombulator.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class InfoBlockRenderer : Microsoft.Xna.Framework.DrawableGameComponent
    {

        internal protected float cellWidth = 16;
        internal protected float cellHeight = 16;
        internal protected KabombulatorGame game;
        private Color fillColor = new Color(255, 0, 0, 64);
        private Color borderColor = Color.White;
        private bool needsInit = true;
        private Vector3 offset;
        private VertexDeclaration decl;
        private VertexBuffer vb;
        private Effects.DiffuseEffect fx;
        private float yaw = 0;
        private float pitch = 0;
        private float roll = 0;
        Services.PlayerManager playerManager;
        private Map map;


        public Map Map
        {
            get { return map; }
            set { map = value; }
        }

        public float Yaw
        {
            get { return yaw; }
            set { yaw = value; }
        }

        public float Pitch
        {
            get { return pitch; }
            set { pitch = value; }
        }

        public float Roll
        {
            get { return roll; }
            set { roll = value; }
        }

        public Vector3 Offset
        {
            get { return offset; }
            set { offset = value; }
        }

        public Color FillColor
        {
            get { return fillColor; }
            set { fillColor = value; }
        }

        public Color BorderColor
        {
            get { return borderColor; }
            set { borderColor = value; }
        }

        public float CellWidth
        {
            get { return cellWidth; }
            set
            {
                cellWidth = value;
                needsInit = true;
            }
        }

        public float CellHeight
        {
            get { return cellHeight; }
            set
            {
                cellHeight = value;
                needsInit = true;
            }
        }

        public void SetCellSize(float w, float h)
        {
            this.cellWidth = w;
            this.cellHeight = h;
            this.needsInit = true;
        }

        public InfoBlockRenderer(KabombulatorGame game, Map map, float cw, float ch)
            : base(game)
        {
            this.game = game;
            this.map = map;
            this.cellWidth = cw;
            this.cellHeight = ch;
            this.playerManager = (Services.PlayerManager)this.game.Services.GetService(typeof(Services.PlayerManager));
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            base.Initialize();
            if (decl == null) decl = new VertexDeclaration(this.GraphicsDevice, VertexPositionColor.VertexElements);
            if (fx == null) fx = new Effects.DiffuseEffect(this.game, this.game.content.Load<Effect>(Content.Shaders));
            // this.camera = this.game.camera;// (Camera)this.game.cam.Clone();
            // this.gridCamera.ProjectionType = Projection.Orthographic;
            if (needsInit) { InitGrid(); }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            if (needsInit) { InitGrid(); }

            base.Update(gameTime);
        }

        private void InitGrid()
        {
            VertexPositionColor[] verts = new VertexPositionColor[9];
            // dispose vb if prev created
            if (vb != null) { vb.Dispose(); vb = null; }
            // create new vb
            vb = new VertexBuffer(this.GraphicsDevice, verts.Length * VertexPositionColor.SizeInBytes, ResourceUsage.None, ResourceManagementMode.Automatic);
            float hw = this.cellWidth / 2;
            float hh = this.cellHeight / 2;

            verts[0] = new VertexPositionColor(new Vector3(-hw, -hh, 0), this.fillColor);
            verts[1] = new VertexPositionColor(new Vector3(hw, -hh, 0), this.fillColor);
            verts[2] = new VertexPositionColor(new Vector3(hw, hh, 0), this.fillColor);
            verts[3] = new VertexPositionColor(new Vector3(-hw, hh, 0), this.fillColor);

            verts[4] = new VertexPositionColor(new Vector3(-hw, -hh, 0), this.borderColor);
            verts[5] = new VertexPositionColor(new Vector3(hw, -hh, 0), this.borderColor);
            verts[6] = new VertexPositionColor(new Vector3(hw, hh, 0), this.borderColor);
            verts[7] = new VertexPositionColor(new Vector3(-hw, hh, 0), this.borderColor);
            verts[8] = verts[4];


            // populate vertex buffer
            vb.SetData<VertexPositionColor>(verts);
            this.needsInit = false;
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            this.map = null;
            this.fx.Dispose();
            this.fx = null;
            this.playerManager = null;
            this.decl.Dispose();
            this.decl = null;
            this.game = null;
            this.vb.Dispose();
            this.vb = null;
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your draw code here
            GraphicsDevice g = this.GraphicsDevice;
            g.VertexDeclaration = decl;
            g.Vertices[0].SetSource(vb, 0, VertexPositionColor.SizeInBytes);


            Matrix baseWorld = Matrix.CreateRotationY(this.yaw) *
                               Matrix.CreateRotationX(this.pitch) *
                               Matrix.CreateRotationZ(this.roll) *
                               Matrix.CreateTranslation(this.offset);
            //fx.World = baseWorld;
            //fx.VertexColorEnabled = true;
            //fx.TextureEnabled = false;

            Viewport vp = this.game.graphics.GraphicsDevice.Viewport;
            foreach (Player p in playerManager.Players)
            {
                if (p.infoBlocksVisible)
                {
                    this.game.graphics.GraphicsDevice.Viewport = p.viewport;
                    //fx.View = p.camera.ViewMatrix;
                    //fx.Projection = p.camera.ProjectionMatrix;

                    fx.Begin(SaveStateMode.SaveState);
                    foreach (EffectPass pass in fx.CurrentTechnique.Passes)
                    {
                        Matrix itemMatrix = Matrix.Identity;

                        for (int idy = 0; idy <= this.map.Height - 1; idy++)
                        {
                            for (int idx = 0; idx <= this.map.Width - 1; idx++)
                            {
                                // make usre index is valid before merging
                                InfoBlock ib;
                                ib = this.map.blocks[idx, idy];
                                if (ib != null && ib.Type != InfoBlock.InfoType.None)
                                {

                                    switch (ib.Type)
                                    {
                                        case InfoBlock.InfoType.ReferenceStructure:
                                            itemMatrix = Matrix.CreateTranslation(ib.Structure.position);
                                            Vector3 relPos;
                                            relPos = new Vector3(ib.XPosition * this.cellWidth, 0, ib.YPosition * this.cellHeight);
                                            itemMatrix *= Matrix.CreateTranslation(relPos);
                                            break;

                                        case InfoBlock.InfoType.Structure:
                                            itemMatrix = Matrix.CreateTranslation(ib.Structure.position);
                                            break;
                                    }

                                    // assign wvp matrix
                                    fx.WorldViewProjection = baseWorld * itemMatrix * p.camera.ViewMatrix * p.camera.ProjectionMatrix;
                                        
                                    // fill square
                                    fx.DiffuseColor = fillColor;
                                    fx.Alpha = this.fillColor.A / 255f;

                                    // begin pass
                                    pass.Begin();
                                    g.DrawPrimitives(PrimitiveType.TriangleFan, 0, 2);
                                    pass.End();

                                    // draw border
                                    fx.DiffuseColor = borderColor;
                                    fx.Alpha = this.borderColor.A / 255f;

                                    // begin pass
                                    pass.Begin();
                                    g.DrawPrimitives(PrimitiveType.LineStrip, 4, 4);
                                    pass.End();

                                }
                            }
                        }




                    }
                    fx.End();
                }
            }
            this.game.graphics.GraphicsDevice.Viewport = vp;

            base.Draw(gameTime);
        }
    }
}


