/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
#endregion

namespace Kabombulator.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class ItemPlacement : Microsoft.Xna.Framework.DrawableGameComponent
    {
        Axiom.Controllers.ControllerManager manager;
        Structures.BaseStructure item;
        List<Controllers.BaseObjectValue> itemvalues = new List<Kabombulator.Controllers.BaseObjectValue>();
        Interfaces.IUserInputService input;
        private Player player;
        private Map map;


        public Map Map
        {
            get { return map; }
            set { map = value; }
        }

        public Player Player
        {
            get { return player; }
            set { player = value; }
        }

        public ItemPlacement(KabombulatorGame game, Player p, Map map)
            : base(game)
        {

            this.player = p;
            this.map = map;

            itemvalues.Add(new Controllers.BaseObjectValue(item, Controllers.BaseObjectValue.XPosition));
            itemvalues.Add(new Controllers.BaseObjectValue(item, Controllers.BaseObjectValue.ZPosition));

            Axiom.Controllers.Canned.AssignmentControllerFunction assign = new Axiom.Controllers.Canned.AssignmentControllerFunction();

            manager = new Axiom.Controllers.ControllerManager(game);
            manager.CreateController(new Controllers.SnapValue(
                                         new Controllers.CameraValue(p.camera, Controllers.CameraValue.XDirection),
                                         KabombulatorGame.GridSize),
                                     itemvalues[0],
                                     assign);
            manager.CreateController(new Controllers.SnapValue(
                                         new Controllers.CameraValue(p.camera, Controllers.CameraValue.ZDirection),
                                         KabombulatorGame.GridSize),
                                     itemvalues[1],
                                     assign);

            this.input = (Interfaces.IUserInputService)this.Game.Services.GetService(typeof(Interfaces.IUserInputService));

        }



        //public void SetObject(BaseObject obj)
        //{
        //    this.item = (DrawableObject)obj;
        //    foreach (Controllers.BaseObjectValue o in itemvalues)
        //    {
        //        o.Object = obj;
        //    }
        //}

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if (input.AClick(player.index) || (player.hasKBControl && input.KeyClick(Keys.Space)))
            {
                if (this.item != null)
                {
                    int xc, yc;
                    InfoBlock[,] blocks = this.item.GetInfoBlocks(out xc, out yc);
                    if (blocks != null)
                    {
                        int xpos, ypos;
                        xpos = (int)(this.item.position.X / KabombulatorGame.GridSize);
                        ypos = (int)(this.item.position.Z / KabombulatorGame.GridSize);
                        xpos += this.map.width / 2;
                        ypos += this.map.height / 2;

                        // if not overlapping blocks then add it
                        if (!this.map.Overlap(blocks, xc, yc, xpos, ypos))
                        {
                            player.objs.Add(this.item);
                            this.map.Merge(blocks, xc, yc, xpos, ypos, false);
                            this.item = null;
                            this.ClearSync();
                        }
                    }
                }
                else
                {
                    Structures.Wall obj = new Kabombulator.Structures.Wall((KabombulatorGame)this.Game);

                    if (player.currency >= obj.cost)
                    {
                        player.currency -= obj.cost;
                        this.item = obj;
                        this.DoSync();
                    }
                    else
                    {
                        obj = null;
                    }
                }
            }

            if (input.LeftSholderClick(player.index) || (player.hasKBControl && input.KeyClick(Keys.Z)))
            {
                if (this.item != null) this.item.RotateLeft();// -= 45;
            }

            if (input.RightSholderClick(player.index) || (player.hasKBControl && input.KeyClick(Keys.X)))
            {
                if (this.item != null) this.item.RotateRight();// += 45;
            }

            if (input.BClick(player.index) && this.item != null)
            {
                player.currency += this.item.cost;
                this.item = null;
            }



            manager.Update(gameTime);



            base.Update(gameTime);
        }

        private void DoSync()
        {
            foreach (Controllers.BaseObjectValue item in this.itemvalues)
            {
                item.Object = this.item;
            }
        }

        private void ClearSync()
        {
            foreach (Controllers.BaseObjectValue item in this.itemvalues)
            {
                item.Object = null;
            }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your update code here

            if (this.item != null)
            {
                this.GraphicsDevice.Viewport = this.player.viewport;
                this.item.Draw(gameTime, this.player.camera);
            }

            base.Draw(gameTime);
        }
    }
}


