
//---- Varibles --------------------------------
float4x4 matWorldViewProj : WorldViewProjection;

texture diffuseTexture : Diffuse
<
	string ResourceName = "default_color.dds";
>;

float4 DiffuseColor : Diffuse
<
    string UIName = "Diffuse Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float Alpha 
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.0;
    float UIStep = 0.01;
    string UIName = "Transparency";
> = 1.0;

float uOffset 
<
    string UIWidget = "slider";
    float UIMin = -10.0;
    float UIMax = 10.0;
    float UIStep = 0.01;
    string UIName = "uOffset";
> = 1.0;

float vOffset 
<
    string UIWidget = "slider";
    float UIMin = -10.0;
    float UIMax = 10.0;
    float UIStep = 0.01;
    string UIName = "vOffset";
> = 1.0;


//---Vertex Structures ---------------------------------
struct vertexInput {
    float3 position				: POSITION;
    float3 normal				: NORMAL;
    float4 texCoordDiffuse		: TEXCOORD0;
};

struct vertexOutput {
    float4 hPosition		: POSITION;
    float4 texCoordDiffuse	: TEXCOORD0;
};


//-----------------------------------
//--- Vertex Shaders -----------------
//-----------------------------------
vertexOutput VS_TransformAndTexture(vertexInput IN) 
{
    vertexOutput OUT;
    OUT.hPosition = mul( float4(IN.position.xyz , 1.0) , matWorldViewProj);
    OUT.texCoordDiffuse = IN.texCoordDiffuse;
	OUT.texCoordDiffuse.x = IN.texCoordDiffuse.x + uOffset;
	OUT.texCoordDiffuse.y = IN.texCoordDiffuse.y + vOffset;

    return OUT;
}

vertexOutput VS_Transform(vertexInput IN) 
{
	vertexOutput OUT;
	OUT.hPosition = mul( float4(IN.position.xyz , 1.0) , matWorldViewProj);
	OUT.texCoordDiffuse = IN.texCoordDiffuse;
	OUT.texCoordDiffuse.x = IN.texCoordDiffuse.x + uOffset;
	OUT.texCoordDiffuse.y = IN.texCoordDiffuse.y + vOffset;

	return OUT;
}

//------------------------------------
sampler TextureSampler = sampler_state 
{
    texture = <diffuseTexture>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER =anisotropic;
    MINFILTER = anisotropic;
    MAGFILTER = anisotropic;
};


//-----------------------------------
//--- Pixel Shaders -----------------
//-----------------------------------

float4 PS_TexturedTransparency( vertexOutput IN): COLOR
{
	float4 diffuseTexture = tex2D( TextureSampler, IN.texCoordDiffuse );
	float4 Col;
	Col = (DiffuseColor * diffuseTexture) * 0.5;
	Col.a = Alpha;
	return Col;
}

float4 PS_Textured( vertexOutput IN): COLOR
{	
	float4 diffuseTexture = tex2D( TextureSampler, IN.texCoordDiffuse );
	return DiffuseColor * diffuseTexture;
}

float4 PS_DiffuseTransparency( vertexOutput IN): COLOR
{
	float4 Col;
	Col = DiffuseColor;
	Col.a = Alpha;
	return Col;
}

float4 PS_Diffuse( vertexOutput IN): COLOR
{
  
    return DiffuseColor;
}

//-----------------------------------
//-- Techniques ---------------------
//-----------------------------------
technique StandardDiffuseTransparency
{
    pass p0 
    {		
    	AlphaBlendEnable = true;
    	srcblend = srcalpha;
    	destblend = srcalpha;
		zwriteenable = false;   
		zenable = false; 
		cullmode = cw;
		zfunc = greater;
		VertexShader = compile vs_1_1 VS_Transform();
		PixelShader  = compile ps_1_1 PS_DiffuseTransparency();
    }
    pass p1 
    {		
    	AlphaBlendEnable = true;
    	srcblend = srcalpha;
    	destblend = srcalpha;
		zwriteenable = true;   
		zenable = true; 		
		cullmode = ccw;
		zfunc = lessequal;
		VertexShader = compile vs_1_1 VS_Transform();
		PixelShader  = compile ps_1_1 PS_DiffuseTransparency();
    }
     
}

technique StandardDiffuse
{
    pass p0 
    {		
    	AlphaBlendEnable = false;
    	srcblend = srcalpha;
    	destblend = srcalpha;
		zwriteenable = true;   
		zenable = true; 
		cullmode = ccw;
		zfunc =lessequal;
		VertexShader = compile vs_1_1 VS_Transform();
		PixelShader  = compile ps_1_1 PS_Diffuse();
    }
     
}

technique TexturedTransparency
{		
    pass p0 
    {		
    	AlphaBlendEnable = true;
    	srcblend = srcalpha;
    	destblend = srcalpha;
		zwriteenable = false;   
		zenable = false; 
		cullmode = cw;
		zfunc = greaterequal;
		VertexShader = compile vs_1_1 VS_TransformAndTexture();
		PixelShader  = compile ps_1_1 PS_TexturedTransparency();
    }
    pass p1 
    {		
    	AlphaBlendEnable = true;
    	srcblend = srcalpha;
    	destblend = srcalpha;
		zwriteenable = true;   
		zenable = true; 
		cullmode = ccw;
		zfunc = lessequal;
		VertexShader = compile vs_1_1 VS_TransformAndTexture();
		PixelShader  = compile ps_1_1 PS_TexturedTransparency();
    }
}

technique Textured
{				   
    pass p0 
    {		
    	AlphaBlendEnable = false;
    	srcblend = srcalpha;
    	destblend = srcalpha;
		zwriteenable = true;   
		zenable = true; 
		cullmode = ccw;
		zfunc = lessequal;
		VertexShader = compile vs_1_1 VS_TransformAndTexture();
		PixelShader  = compile ps_1_1 PS_Textured();
    }      
}