/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Kabombulator.Controllers
{
    public class CameraValue : Axiom.Controllers.IControllerValue, IDisposable
    {
        public const int XPosition = 0;
        public const int YPosition = 1;
        public const int ZPosition = 2;
        public const int XDirection = 3;
        public const int YDirection = 4;
        public const int ZDirection = 5;
        public const int XOrientation = 6;
        public const int YOrientation = 7;
        public const int ZOrientation = 8;
        public const int Near = 9;
        public const int Far = 10;
        public const int FOV = 11;
        private int source;
        private Camera camera;


        public Camera Camera
        {
            get { return camera; }
            set { camera = value; }
        }

        public int Source
        {
            get { return source; }
            set { source = value; }
        }

        public CameraValue(Camera cam, int cameraSource)
        {
            this.camera = cam;
            this.source = cameraSource;
        }


        #region IControllerValue Members

        public float Value
        {
            get
            {
                switch (source)
                {
                    case XPosition:
                        return camera != null ? camera.position.X : 0;
                                                       
                    case YPosition:                    
                        return camera != null ? camera.position.Y : 0;
                                                       
                    case ZPosition:                    
                        return camera != null ? camera.position.Z : 0;
                                                       
                    case XDirection:                   
                        return camera != null ? camera.direction.X  : 0;
                                                       
                    case YDirection:                   
                        return camera != null ? camera.direction.Y  : 0;
                                                       
                    case ZDirection:                   
                        return camera != null ? camera.direction.Z  : 0;
                                                                   
                    case XOrientation:                 
                        return camera != null ? camera.orientation.X : 0;
                                                       
                    case YOrientation:                 
                        return camera != null ? camera.orientation.Y : 0;
                                                       
                    case ZOrientation:                 
                        return camera != null ? camera.orientation.Z : 0;
                                                       
                    case Near:                         
                        return camera != null ? camera.nearDistance : 0;
                                                       
                    case Far:                          
                        return camera != null ? camera.farDistance  : 0;
                                                       
                    case FOV:                          
                        return camera != null ? camera.fieldOfView  : 0;
                }

                return 0;

            }
            set
            {
                switch (source)
                {
                    case XPosition:
                        if (camera != null) camera.position.X = value;
                        break;

                    case YPosition:
                        if (camera != null) camera.position.Y =   value;
                        break;

                    case ZPosition:
                        if (camera != null) camera.position.Z =  value;
                        break;

                    case XDirection:
                        if (camera != null) camera.direction.X =  value;
                        break;

                    case YDirection:
                        if (camera != null) camera.direction.Y = value;
                        break;

                    case ZDirection:
                        if (camera != null) camera.direction.Z =  value;
                        break;
                                                                
                    case XOrientation:
                        if (camera != null) camera.orientation.X =  value;
                        break;

                    case YOrientation:
                        if (camera != null) camera.orientation.Y = value;
                        break;

                    case ZOrientation:
                        if (camera != null) camera.orientation.Z = value;
                        break;

                    case Near:
                        if (camera != null) camera.nearDistance = value;
                        break;

                    case Far:
                        if (camera != null) camera.farDistance = value;
                        break;

                    case FOV:
                        if (camera != null) camera.fieldOfView = value;
                        break;
                }

            }
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            this.camera = null;
        }

        #endregion
    }
}
