/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Kabombulator.Effects
{
    public class DiffuseEffect : Effect
    {
        protected KabombulatorGame game;
        protected Matrix wvp = Matrix.Identity;
        protected float alpha = 255;
        protected Vector4 diffuse = Color.White.ToVector4();
        protected bool alphaEnabled = false;


        public virtual bool EnableAlpha
        {
            get { return alphaEnabled; }
            set { 
                alphaEnabled = value;
                if (this.alphaEnabled)
                {
                    this.CurrentTechnique = this.Techniques["StandardDiffuseTransparency"];
                }
                else
                {
                    this.CurrentTechnique = this.Techniques["StandardDiffuse"];

                }
            }
        }        

        public Color DiffuseColor
        {
            get { return new Color( this.diffuse); }
            set
            {
                this.diffuse = value.ToVector4();
                this.Parameters["DiffuseColor"].SetValue(this.diffuse);
            }
        }

        public Matrix WorldViewProjection
        {
            get { return this.wvp; }
            set
            {
                this.wvp = value;
                this.Parameters["matWorldViewProj"].SetValue(value);
            }
        }

        public float Alpha
        {
            get { return this.alpha; }
            set
            {
                this.alpha = value;
                this.Parameters["Alpha"].SetValue(value);
            }
        }

        public KabombulatorGame Game
        {
            get { return game; }
        }

        public DiffuseEffect(KabombulatorGame game, Effect cloneSource)
            : base(game.graphics.GraphicsDevice, cloneSource)
        {
            this.game = game;
            this.CurrentTechnique = this.Techniques["StandardDiffuse"];

            this.alpha = this.Parameters["Alpha"].GetValueSingle();
            this.wvp = this.Parameters["matWorldViewProj"].GetValueMatrix();
            this.diffuse = this.Parameters["DiffuseColor"].GetValueVector4();
            
        }

    }
}
