/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
//#define TwoPlayer

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Kabombulator.Properties;
#endregion

namespace Kabombulator
{

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class KabombulatorGame : Microsoft.Xna.Framework.Game
    {


        protected internal GraphicsDeviceManager graphics;
        protected internal ContentManager content;
        protected internal Collections.ModelDictionary models;
        public const int DefaultStartCurrency = 1000;
        public const float MinCameraDistance = 175f;
        public const float MaxCameraDistance = 600f;
        public const float GridSize = 25f;
        public const int MapSize = 10000;
        private Map map;
        Services.PlayerManager playerManager;
        private Nuclex.Fonts.TextManager textManager;
        protected internal Interfaces.IUserInputService input;
        private Nuclex.Fonts.BitmapFont gameFont;



        public Nuclex.Fonts.BitmapFont GameFont
        {
            get { return gameFont; }
            set { gameFont = value; }
        }

        public Nuclex.Fonts.TextManager TextManager
        {
            get { return this.textManager; }
        }

        public Map Map
        {
            get { return map; }
        }

        public KabombulatorGame()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            this.Window.AllowUserResizing = true;
            this.graphics.PreferMultiSampling = true;
            models = new Kabombulator.Collections.ModelDictionary();

            // add input service
            this.input = new Kabombulator.Services.UserInputService(this);
            this.Services.AddService(typeof(Interfaces.IUserInputService), this.input);

            // add player manager
            this.playerManager = new Services.PlayerManager(this);
            this.Services.AddService(typeof(Services.PlayerManager), playerManager);

            // init map
            int size = (int)(MapSize / GridSize);
            this.map = new Map(this, size, size);

            // add first player to players list
            Player playerOne = new Player(this);
            playerOne.PlayerIndex = PlayerIndex.One;
            playerOne.Type = Player.PlayerType.Local;
            playerOne.Viewport = this.graphics.GraphicsDevice.Viewport;
#if TwoPlayer
            playerOne.HasKeyboardControl = false;
            playerOne.HasMouseControl = false;
#else
            playerOne.HasKeyboardControl = true;
            playerOne.HasMouseControl = true;
#endif
            playerOne.currency = DefaultStartCurrency;
            playerManager.Players.Add(playerOne);

#if TwoPlayer
            // add second player to players list
            Player playerTwo = new Player(this);
            playerTwo.PlayerIndex = PlayerIndex.Two;
            playerTwo.Type = Player.PlayerType.Local;
            playerTwo.Viewport = this.graphics.GraphicsDevice.Viewport;
            playerTwo.HasKeyboardControl = true;
            playerTwo.HasMouseControl = true;
            playerTwo.currency = DefaultStartCurrency;
            playerManager.Players.Add(playerTwo);
#endif

            // add grid rendering component
            Kabombulator.Components.GridRenderer gr = new Components.GridRenderer(this, MapSize, MapSize);
            gr.CellWidth = GridSize;
            gr.CellHeight = GridSize;
            gr.Pitch = MathHelper.PiOver2;
            Vector3 offset = new Vector3(gr.gridWidth * 0.5f, 0, gr.gridHeight * 0.5f) - new Vector3(gr.cellWidth * 0.5f, 0, gr.cellHeight * 0.5f);
            gr.Offset = (Vector3.Down * 0.5f) + offset;
            this.Components.Add(gr);

            // add info block rendering component
            Kabombulator.Components.InfoBlockRenderer ibr = new Components.InfoBlockRenderer(this, this.map, GridSize, GridSize);
            ibr.Pitch = MathHelper.PiOver2;
            // offset = new Vector3(MapSize * 0.5f, 0, MapSize * 0.5f) - new Vector3(gr.cellWidth * 0.5f, 0, gr.cellHeight * 0.5f);
            // offset = new Vector3(MapSize * 0.5f, 0, MapSize * 0.5f) - new Vector3(gr.cellWidth * 0.5f, 0, gr.cellHeight * 0.5f);
            offset = Vector3.Zero;
            ibr.Offset = (Vector3.Up * 1f) + offset;
            this.Components.Add(ibr);

            // add an item placement compoent for player one
            Kabombulator.Components.ItemPlacement ip = new Kabombulator.Components.ItemPlacement(this, playerOne, this.map);
            this.Components.Add(ip);

#if TwoPlayer
            // add an item placement compoent for player two
            ip = new Kabombulator.Components.ItemPlacement(this, playerTwo,this.map);
            this.Components.Add(ip);
#endif


            textManager = new Nuclex.Fonts.TextManager(this);
            this.Components.Add(textManager);
            this.Services.AddService(typeof(Nuclex.Fonts.TextManager), textManager);

            this.Window.ClientSizeChanged += new EventHandler(Window_ClientSizeChanged);
            Window_ClientSizeChanged(this, null);

            base.Initialize();
        }

        void Window_ClientSizeChanged(object sender, EventArgs e)
        {
            int lpcount = playerManager.LocalPlayerCount;
            foreach (Player p in playerManager.Players)
            {
                if (p.Type == Player.PlayerType.Local)
                {
                    switch (lpcount)
                    {
                        case 1:
                            if (p.index == PlayerIndex.One)
                            {
                                p.viewport.X = 0;
                                p.viewport.Y = 0;
                                p.viewport.Width = this.Window.ClientBounds.Width;
                                p.viewport.Height = this.Window.ClientBounds.Height;
                            }
                            break;

                        case 2:
                            switch (p.index)
                            {
                                case PlayerIndex.One:
                                    p.viewport.X = 0;
                                    p.viewport.Y = 0;
                                    p.viewport.Width = this.Window.ClientBounds.Width / 2;
                                    p.viewport.Height = this.Window.ClientBounds.Height / 2;
                                    break;

                                case PlayerIndex.Two:
                                    p.viewport.Width = this.Window.ClientBounds.Width / 2;
                                    p.viewport.Height = this.Window.ClientBounds.Height / 2;
                                    p.viewport.X = p.viewport.Width;
                                    p.viewport.Y = p.viewport.Height;
                                    break;
                            }
                            break;

                        case 3:
                            switch (p.index)
                            {
                                case PlayerIndex.One:
                                    p.viewport.X = 0;
                                    p.viewport.Y = 0;
                                    p.viewport.Width = this.Window.ClientBounds.Width / 3;
                                    p.viewport.Height = this.Window.ClientBounds.Height;
                                    break;

                                case PlayerIndex.Two:
                                    p.viewport.Width = this.Window.ClientBounds.Width / 3;
                                    p.viewport.Height = this.Window.ClientBounds.Height;
                                    p.viewport.X = p.viewport.Width;
                                    p.viewport.Y = 0;
                                    break;

                                case PlayerIndex.Three:
                                    p.viewport.Width = this.Window.ClientBounds.Width / 3;
                                    p.viewport.Height = this.Window.ClientBounds.Height;
                                    p.viewport.X = p.viewport.Width * 2;
                                    p.viewport.Y = 0;
                                    break;
                            }
                            break;

                        case 4:
                            switch (p.index)
                            {
                                case PlayerIndex.One:
                                    p.viewport.X = 0;
                                    p.viewport.Y = 0;
                                    p.viewport.Width = this.Window.ClientBounds.Width / 2;
                                    p.viewport.Height = this.Window.ClientBounds.Height / 2;
                                    break;

                                case PlayerIndex.Two:
                                    p.viewport.Width = this.Window.ClientBounds.Width / 2;
                                    p.viewport.Height = this.Window.ClientBounds.Height / 2;
                                    p.viewport.X = p.viewport.Width;
                                    p.viewport.Y = 0;
                                    break;

                                case PlayerIndex.Three:
                                    p.viewport.Width = this.Window.ClientBounds.Width / 2;
                                    p.viewport.Height = this.Window.ClientBounds.Height / 2;
                                    p.viewport.X = 0;
                                    p.viewport.Y = p.viewport.Height;
                                    break;

                                case PlayerIndex.Four:
                                    p.viewport.Width = this.Window.ClientBounds.Width / 2;
                                    p.viewport.Height = this.Window.ClientBounds.Height / 2;
                                    p.viewport.X = p.viewport.Width;
                                    p.viewport.Y = p.viewport.Height;
                                    break;
                            }
                            break;

                    }
                }
            }
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                models.Add(Content.Wall, content.Load<Model>(Content.Wall));
                models.Add(Content.GrassyGround, content.Load<Model>(Content.GrassyGround));
                models.Add(Content.PowerTower, content.Load<Model>(Content.PowerTower));

                // modify the ground model to use a TextureEffect BasicEffect has numerous "issues"
                Effects.TexturedEffect fx = new Kabombulator.Effects.TexturedEffect(this, content.Load<Effect>(Content.Shaders));
                Model m = models[Content.GrassyGround];
                BasicEffect be = (BasicEffect)(m.Meshes[0].MeshParts[0].Effect);
                fx.Texture = be.Texture;
                m.Meshes[0].MeshParts[0].Effect = fx;

                this.gameFont = content.Load<Nuclex.Fonts.BitmapFont>(Content.prstart);
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
                models.Clear();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            input.Update(gameTime);
            playerManager.Update(gameTime);

            base.Update(gameTime);
        }




        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);
            Viewport vp = this.graphics.GraphicsDevice.Viewport;

            // draw any player views
            foreach (Player p in playerManager.Players)
            {
                // only draw the view if the playre is a local player
                if (p.Type == Player.PlayerType.Local) DrawPlayerView(gameTime, p);
            }


            base.Draw(gameTime);
            this.graphics.GraphicsDevice.Viewport = vp;
        }

        private void DrawPlayerView(GameTime gameTime, Player p)
        {
            this.graphics.GraphicsDevice.Viewport = p.viewport;
            // draw the ground
            Helpers.DrawModel(gameTime, p.camera, models[Content.GrassyGround]);

            // draw players objcets
            foreach (DrawableObject o in p.objs)
            {
                Helpers.DrawModel(gameTime, p.camera, o.model, o.position, new Vector3(0, o.Rotation, 0), o.color);
            }

            // draw player currency
            textManager.DrawString(this.gameFont, new Vector2(p.viewport.X, p.viewport.Y),
                                   Resources.Currency + p.currency.ToString(),
                                   Nuclex.Fonts.TextAlignment.Top | Nuclex.Fonts.TextAlignment.Left,
                                   Color.White);

            // draw structure building icon/info

        }
    }
}