/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Kabombulator
{
    public class Map
    {
       protected internal InfoBlock[,] blocks;
        protected internal KabombulatorGame game;
        protected internal int width;
        protected internal int height;
        protected internal bool needsInit = true;


        public bool NeedsInitilizing
        {
            get { return needsInit; }
        }

        public int Height
        {
            get { return height; }
            set { height = value; }
        }

        public int Width
        {
            get { return width; }
            set { width = value; }
        }


        public KabombulatorGame Game
        {
            get { return game; }
        }


        public Map(KabombulatorGame game, int w, int h)
        {
            this.game = game;
            this.Width = w;
            this.Height = h;
            this.InitMap();
        }

        private void InitMap()
        {
            this.blocks = new InfoBlock[this.width, this.height];
        }

        public bool Overlap(InfoBlock[,] items, int xCenter, int yCenter, int xIndex, int yIndex)
        {
            int x, y;
            for (int idy = 0; idy <= items.GetUpperBound(1); idy++)
            {
                for (int idx = 0; idx <= items.GetUpperBound(0); idx++)
                {
                    x = (xIndex + idx) - xCenter;
                    y = (yIndex + idy) - yCenter;

                    // make usre index is valid before merging
                    if (x >= 0 && x <= this.blocks.GetUpperBound(0) &&
                        y >= 0 && y <= this.blocks.GetUpperBound(1) &&
                        this.blocks[x, y] != null && 
                        items[idx, idy] != null && items[idx, idy].Type != InfoBlock.InfoType.None )
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        public void Merge(InfoBlock[,] items, int xCenter, int yCenter, int xIndex, int yIndex, bool replace)
        {
            for (int idy = 0; idy <= items.GetUpperBound(1); idy++)
            {
                for (int idx = 0; idx <= items.GetUpperBound(0); idx++)
                {
                    int x, y;
                    x = (xIndex + idx) - xCenter;
                    y = (yIndex + idy) - yCenter;

                    // make usre index is valid before merging
                    if (x >= 0 && x <= this.blocks.GetUpperBound(0) &&
                        y >= 0 && y <= this.blocks.GetUpperBound(1))
                    {
                        // only merge block if not already present
                        if (!replace && this.blocks[x, y] == null)
                        {
                            this.blocks[x, y] = null;
                            this.blocks[x, y] = items[idx, idy];
                        }
                    }
                }
            }
        }
    }
}
