/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;

namespace Kabombulator
{
    public class Player : IDisposable
    {
        public enum PlayerType
        {
            Local = 0,
            Computer = 1,
            Remote = 2
        }

        public enum PlayerState
        {
            Playing,
            SelectingStructure
        }

        protected internal string name;
        protected internal int score;
        protected internal Collections.DrawableObjectCollection objs = new Kabombulator.Collections.DrawableObjectCollection();
        protected internal PlayerType playerType = PlayerType.Local;
        protected internal int currency = 0;
        protected internal Viewport viewport;
        protected internal KabombulatorGame game;
        protected internal PlayerIndex index = PlayerIndex.One;
        protected internal bool hasKBControl = false;
        protected internal bool hasMouseControl = false;
        protected internal Camera camera;
        protected internal bool gridVisible = true;
        protected internal bool infoBlocksVisible = true;
        protected internal Interfaces.IUserInputService input;
        protected internal PlayerState state = PlayerState.Playing;



        public PlayerState State
        {
            get { return state; }
            set { state = value; }
        }

        public bool InfoBlocksVisible
        {
            get { return infoBlocksVisible; }
            set { infoBlocksVisible = value; }
        }

        public bool GridVisible
        {
            get { return gridVisible; }
            set { gridVisible = value; }
        }


        public bool HasMouseControl
        {
            get { return hasMouseControl; }
            set
            {
                if (this.playerType == PlayerType.Local)
                {
                    hasMouseControl = value;
                }
                else
                {
                    hasMouseControl = false;
                }
            }
        }

        public bool HasKeyboardControl
        {
            get { return hasKBControl; }
            set
            {
                if (this.playerType == PlayerType.Local)
                {
                    hasKBControl = value;
                }
                else
                {
                    hasKBControl = false;
                }
            }
        }

        public void Update(GameTime gameTime)
        {
            HandleInput(gameTime);
            camera.Update(gameTime);
        }

        private void HandleInput(GameTime gameTime)
        {
            HandleGamePadInput(gameTime);

            if (hasKBControl) HandleKeyboardInput(gameTime);
            if (HasMouseControl) HandleMouseInput(gameTime);
        }

        private void HandleMouseInput(GameTime gameTime)
        {
            // throw new Exception("The method or operation is not implemented.");
        }

        private void HandleKeyboardInput(GameTime gameTime)
        {

            if (input.KeyClick(Keys.F12)) this.game.graphics.ToggleFullScreen();
            if (input.KeyClick(Keys.Escape)) this.game.Exit();

            // toggle grid / info blocks
            if (input.KeyClick(Keys.G)) this.gridVisible = !this.gridVisible;
            if (input.KeyClick(Keys.B)) this.infoBlocksVisible = !this.infoBlocksVisible;

            // get rotation
            float lt = input.IsKeyDown(Keys.Q) ? 1 : 0;
            float rt = input.IsKeyDown(Keys.E) ? 1 : 0;
            float rotation = (-lt + rt) * gameTime.ElapsedGameTime.Milliseconds;

            // get panning
            float val = 1f * gameTime.ElapsedGameTime.Milliseconds;
            Vector2 Horiz = new Vector2((input.IsKeyDown(Keys.A) ? -val : 0) + (input.IsKeyDown(Keys.D) ? val : 0), 0);
            Vector2 Vert = Vector2.Zero;

            if (input.IsKeyUp(Keys.LeftShift))
            {
                Vert = new Vector2(0, (input.IsKeyDown(Keys.W) ? val : 0) + -(input.IsKeyDown(Keys.S) ? val : 0));
            }
            Vector2 pan = Horiz + Vert;

            // get zoom
            val = 0;
            if (input.IsKeyDown(Keys.LeftShift))
            {
                if (input.IsKeyDown(Keys.W)) val = -0.75f;
                if (input.IsKeyDown(Keys.S)) val = 0.75f;
            }
            val *= gameTime.ElapsedGameTime.Milliseconds;

            UpdateCamera(gameTime, pan, rotation, val);
        }

        private void HandleGamePadInput(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (input.BackClick(this.index)) this.game.Exit();

            // toggle grid
            if (input.RightStickClick(this.index)) this.gridVisible = !this.gridVisible;

            // toggle info blocks
            if (input.LeftStickClick(this.index)) this.infoBlocksVisible = !this.infoBlocksVisible;

            Vector2 v2 = input.LeftThumbStick(this.index) * (0.75f * gameTime.ElapsedGameTime.Milliseconds);
            float lt = input.LeftTrigger(this.index);
            float rt = input.RightTrigger(this.index);
            float rotation = (lt + -rt) * gameTime.ElapsedGameTime.Milliseconds;

            // get zoom
            float val = 0;
            if (input.Up(this.index) == ButtonState.Pressed) val = -0.75f;
            if (input.Down(this.index) == ButtonState.Pressed) val = 0.75f;
            val *= gameTime.ElapsedGameTime.Milliseconds;

            UpdateCamera(gameTime, v2, rotation, val);
        }

        private void UpdateCamera(GameTime gameTime, Vector2 v2, float rotation, float zoom)
        {
            Vector3 pos = this.camera.Position;
            Vector3 dir = this.camera.Direction;

            v2.Y = -v2.Y;


            float direction = Helpers.GetRotationInRadians(pos.X, pos.Z, dir.X, dir.Z);
            float offsetdirection = Helpers.GetRotationInRadians(0, 0, v2.X, v2.Y);

            Vector3 newpos = Vector3.Zero;

            // ========= perform zoom ===============
            float val;// = 0.5f * gameTime.ElapsedGameTime.Milliseconds; 
            float dist = Vector2.Distance(new Vector2(dir.X, dir.Z), new Vector2(pos.X, pos.Z));
            dist += zoom;// input.Up(this.index) == ButtonState.Pressed ? -val : 0;
            //dist += input.Down(this.index) == ButtonState.Pressed ? val : 0;

            // cap distances
            dist = dist < KabombulatorGame.MinCameraDistance ? KabombulatorGame.MinCameraDistance : dist;
            dist = dist > KabombulatorGame.MaxCameraDistance ? KabombulatorGame.MaxCameraDistance : dist;

            //newpos = pos - dir;
            //newpos.Normalize();
            //newpos *= dist;
            newpos.X = dir.X + ((float)Math.Cos(-direction) * dist);
            newpos.Z = dir.Z + ((float)Math.Sin(-direction) * dist);
            newpos.Y = dir.Y + dist;// (dist * 0.75f);
            pos = newpos;

            // ========== perform panning ==========
            newpos.X = (float)Math.Cos(direction + offsetdirection + MathHelper.PiOver2);
            newpos.Z = (float)Math.Sin(direction + offsetdirection + MathHelper.PiOver2);
            newpos.Y = 0;

            dist = Vector2.Distance(Vector2.Zero, v2);

            newpos *= (dist * (pos.Y * 0.0025f));
            dir += newpos;
            pos += newpos;

            // ========== perform rotation =======
            val = (rotation * ((MathHelper.Pi / 180f) * 0.25f));

            dist = Vector2.Distance(new Vector2(dir.X, dir.Z), new Vector2(pos.X, pos.Z));
            pos.X = dir.X + ((float)Math.Cos(-direction + -val) * dist);
            pos.Z = dir.Z + ((float)Math.Sin(-direction + -val) * dist);

            //newpos.Y = pos.Y;
            //pos = newpos;

            this.camera.Direction = dir;
            this.camera.Position = pos;
        }

        //private void MoveCamera(GameTime gameTime, Vector2 v2, float lt, float rt)
        //{
        //    Vector3 pos = this.camera.Position;
        //    Vector3 dir = this.camera.Direction;

        //    float val = (lt + -rt) * gameTime.ElapsedGameTime.Milliseconds;

        //    float dist = Math.Abs(dir.Z - pos.Z);

        //    dist = (dist + val) < KabombulatorGame.MinCameraDistance ? KabombulatorGame.MinCameraDistance : dist += val;
        //    dist = dist > KabombulatorGame.MaxCameraDistance ? KabombulatorGame.MaxCameraDistance : dist;

        //    float percent = (dist / 2) * 0.01f; // one percent of half the distance
        //    Vector3 newpos = new Vector3(v2.X, 0, -v2.Y) * gameTime.ElapsedGameTime.Milliseconds;
        //    dir += (newpos * percent);
        //    pos += (newpos * percent);


        //    pos.Z = dir.Z + dist;
        //    pos.Y = dist * 2;

        //    this.camera.Direction = dir;
        //    this.camera.Position = pos;
        //}

        public PlayerIndex PlayerIndex
        {
            get { return index; }
            set { index = value; }
        }

        public KabombulatorGame Game
        {
            get { return game; }
        }

        public Player(KabombulatorGame game)
        {
            this.game = game;
            this.camera = new Camera(game);
            this.input = (Interfaces.IUserInputService)this.game.Services.GetService(typeof(Interfaces.IUserInputService));
        }

        public Viewport Viewport
        {
            get { return viewport; }
            set { viewport = value; }
        }

        public int Currency
        {
            get { return currency; }
            set { currency = value; }
        }

        public PlayerType Type
        {
            get { return playerType; }
            set { playerType = value; }
        }

        public Collections.DrawableObjectCollection Objects
        {
            get { return objs; }
        }

        public int Score
        {
            get { return score; }
            set { score = value; }
        }

        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        #region IDisposable Members

        public void Dispose()
        {
            objs.Dispose();
            objs = null;
        }

        #endregion
    }
}
