/*
Created by: X
http://www.createdbyx.com/

Kabombulator is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Kabombulator.Structures
{
    public class BaseStructure : DrawableObject
    {
        protected internal int level = 1;
        protected internal int health = -1;
        protected internal int maxHealth = -1;
        protected internal int cost = 0;
        protected internal string name = string.Empty;



        public virtual string Name
        {
            get { return name; }
            set { name = value; }
        }                        	

        public override object Clone()
        {
            BaseStructure obj = (BaseStructure)base.Clone();
            obj.level = this.level;
            obj.health = this.health;
            obj.maxHealth = this.maxHealth;
            obj.cost = this.cost;
            return obj;
        }

        public InfoBlock[,] BuildInfoBlocks( int w, int h, int xCenter, int yCenter)
        {
            InfoBlock[,] items = null;
            items = new InfoBlock[w, h];
            for (int idy = 0; idy <= items.GetUpperBound(1); idy++)
            {
                for (int idx = 0; idx <= items.GetUpperBound(0); idx++)
                {
                    InfoBlock ib = new InfoBlock(this, InfoBlock.InfoType.ReferenceStructure);
                    items[idx, idy] = ib;
                    ib.XPosition = idx - xCenter;
                    ib.YPosition = idy - yCenter;
                }
            }

            items[xCenter, yCenter] = new InfoBlock(this, InfoBlock.InfoType.Structure);

            return items;
        }

        public virtual int Cost
        {
            get { return cost; }
            set { cost = value; }
        }

        public virtual int MaxHealth
        {
            get { return maxHealth; }
            set
            {
                maxHealth = value;
                this.Health = this.health;
            }
        }

        public virtual int Health
        {
            get { return health; }
            set
            {
                health = (health + value) > maxHealth ? maxHealth : value;
                health = health < 0 ? 0 : health;
            }
        }

        public virtual int Level
        {
            get { return level; }
            set { level = value; }
        }

        public BaseStructure(KabombulatorGame game)
            : base(game)
        {
        }
    }
}
